/*
 *    XMLConverter.java
 *    Copyright (C) 2002 New Zealand Digital Library, http://www.nzdl.org
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.greenstone.gsdl3.util;

// XML classes
import org.w3c.dom.Document; 
import org.w3c.dom.Node; 
import org.xml.sax.InputSource;
import javax.xml.parsers.*;
import org.apache.xindice.xml.TextWriter;
import org.apache.xerces.dom.TextImpl;
import org.apache.xerces.parsers.DOMParser;

// other java classes
import java.io.Reader;
import java.io.StringReader;
import java.io.File;
import java.io.FileReader;

/** XMLConverter - utitlity class for greenstone
 *
 * parses XML Strings into Documents, converts Nodes to Strings
 * different parsers have different behaviour - can experiment in here
 *
 * @author <a href="mailto:kjdon@cs.waikato.ac.nz">Katherine Don</a>
 * @version $Revision: 3235 $
 *
 */
public class XMLConverter {

    /** JAXP parser factory */
    protected DocumentBuilderFactory doc_build_fact_=null;
    /** JAXP parser */
     protected DocumentBuilder doc_builder_=null;

    /** xerces parser */
    protected DOMParser parser_ = null;

     /** the no-args constructor */
    public XMLConverter() {
	try {
	     doc_build_fact_ = DocumentBuilderFactory.newInstance();
	    
	     doc_builder_ = doc_build_fact_.newDocumentBuilder(); 
	    parser_ = new DOMParser();
	} catch (Exception e) {
	    System.out.println("XMLConverter:exception "+e.getMessage());
	}
    }

    /** returns a DOM Document */
    public Document getDOM(String in) {
	    
	try {
	    Reader reader = new StringReader(in);
	    InputSource xml_source = new InputSource(reader);
	    
	    // Document doc = doc_builder_.parse(xml_source);
	    parser_.parse(xml_source);
	    Document doc = parser_.getDocument();
	    return doc;

	} catch (Exception e) {
	    System.out.println("XMLConverter.getDOM(String): exception "+e.getMessage());
	}
	return null;
    }
  
    /** returns a DOM Document */
    public Document getDOM(File in) {
	try {
	    
	    Reader reader = new FileReader(in);
	    InputSource xml_source = new InputSource(reader);
	    
	    //Document doc = doc_builder_.parse(xml_source);
	    parser_.parse(xml_source);
	    Document doc = parser_.getDocument();
	    
	    return doc;

	} catch (Exception e) {
	    System.out.println("XMLConverter.getDOM(File): exception "+e.getMessage());
	}
	return null;
    }

    /** creates a new empty DOM Document */
    public Document newDOM() {
	Document doc = doc_builder_.newDocument();
	return doc;
    }

    /** returns the Node as a String */
    public String getString(Node doc) {
	TextWriter t = new TextWriter(doc);
	return t.toString();
    }
}
